import re


EXAMPLE_PARSER = re.compile(r'(\w+)\s+([^\n]*)\n?(.*)?\n(\d+)\n(.*)', re.DOTALL)


class Example:
    """
    Apipie Example
    """

    def __init__(self, http_method, path, args, status, response):
        self.http_method = http_method
        self.path = path
        self.args = args
        self.status = int(status)
        self.response = response

    @classmethod
    def parse(cls, example):
        parsed = EXAMPLE_PARSER.match(example)
        return cls(*parsed.groups())



# Based on ActiveSupport Inflector (https://github.com/rails/rails.git)
# Inflection rules taken from davidcelis's Inflections (https://github.com/davidcelis/inflections.git)

import re


class Inflections:

    def __init__(self):
        self.plurals = []
        self.singulars = []
        self.uncountables = []
        self.humans = []
        self.acronyms = {}
        self.acronym_regex = r'/(?=a)b/'

    def acronym(self, word):
        self.acronyms[word.lower()] = word
        self.acronym_regex = '|'.join(self.acronyms.values())

    def plural(self, rule, replacement):
        if rule in self.uncountables:
            self.uncountables.remove(rule)
        if replacement in self.uncountables:
            self.uncountables.remove(replacement)

        self.plurals.insert(0, (rule, replacement))

    def singular(self, rule, replacement):
        if rule in self.uncountables:
            self.uncountables.remove(rule)
        if replacement in self.uncountables:
            self.uncountables.remove(replacement)

        self.singulars.insert(0, (rule, replacement))

    def irregular(self, singular, plural):
        if singular in self.uncountables:
            self.uncountables.remove(singular)
        if plural in self.uncountables:
            self.uncountables.remove(plural)

        s0 = singular[0]
        srest = singular[1:]

        p0 = plural[0]
        prest = plural[1:]

        if s0.upper() == p0.upper():
            self.plural(r'(?i)({}){}$'.format(s0, srest), r'\1' + prest)
            self.plural(r'(?i)({}){}$'.format(p0, prest), r'\1' + prest)

            self.singular(r'(?i)({}){}$'.format(s0, srest), r'\1' + srest)
            self.singular(r'(?i)({}){}$'.format(p0, prest), r'\1' + srest)
        else:
            self.plural(r'{}(?i){}$'.format(s0.upper(), srest), p0.upper() + prest)
            self.plural(r'{}(?i){}$'.format(s0.lower(), srest), p0.lower() + prest)
            self.plural(r'{}(?i){}$'.format(p0.upper(), prest), p0.upper() + prest)
            self.plural(r'{}(?i){}$'.format(p0.lower(), prest), p0.lower() + prest)

            self.singular(r'{}(?i){}$'.format(s0.upper(), srest), s0.upper() + srest)
            self.singular(r'{}(?i){}$'.format(s0.lower(), srest), s0.lower() + srest)
            self.singular(r'{}(?i){}$'.format(p0.upper(), prest), s0.upper() + srest)
            self.singular(r'{}(?i){}$'.format(p0.lower(), prest), s0.lower() + srest)

    def uncountable(self, *words):
        self.uncountables.extend(words)

    def human(self, rule, replacement):
        self.humans.insert(0, (rule, replacement))


class Inflector:

    def __init__(self):
        self.inflections = Inflections()
        self.inflections.plural(r'$', 's')
        self.inflections.plural(r'(?i)([sxz]|[cs]h)$', r'\1es')
        self.inflections.plural(r'(?i)([^aeiouy]o)$', r'\1es')
        self.inflections.plural(r'(?i)([^aeiouy])y$', r'\1ies')

        self.inflections.singular(r'(?i)s$', r'')
        self.inflections.singular(r'(?i)(ss)$', r'\1')
        self.inflections.singular(r'([sxz]|[cs]h)es$', r'\1')
        self.inflections.singular(r'([^aeiouy]o)es$', r'\1')
        self.inflections.singular(r'(?i)([^aeiouy])ies$', r'\1y')

        self.inflections.irregular('child', 'children')
        self.inflections.irregular('man', 'men')
        self.inflections.irregular('medium', 'media')
        self.inflections.irregular('move', 'moves')
        self.inflections.irregular('person', 'people')
        self.inflections.irregular('self', 'selves')
        self.inflections.irregular('sex', 'sexes')

        self.inflections.uncountable('equipment', 'information', 'money', 'species', 'series', 'fish', 'sheep', 'police')

    def pluralize(self, word):
        return self._apply_inflections(word, self.inflections.plurals)

    def singularize(self, word):
        return self._apply_inflections(word, self.inflections.singulars)

    def _apply_inflections(self, word, rules):
        result = word

        if word != '' and result.lower() not in self.inflections.uncountables:
            for (rule, replacement) in rules:
                result = re.sub(rule, replacement, result)
                if result != word:
                    break

        return result



import re

HTML_STRIP = re.compile(r'<\/?[^>]+?>')


class Param:
    """
    Apipie Param
    """

    def __init__(self, **kwargs):
        self.allow_nil = kwargs.get('allow_nil')
        self.description = HTML_STRIP.sub('', kwargs.get('description'))
        self.expected_type = kwargs.get('expected_type')
        self.full_name = kwargs.get('full_name')
        self.name = kwargs.get('name')
        self.params = [Param(**param) for param in kwargs.get('params', [])]
        self.required = bool(kwargs.get('required'))
        self.validator = kwargs.get('validator')



"""
setup.py for apypie
"""
from os import path
from io import open
# Always prefer setuptools over distutils
from setuptools import setup, find_packages

# Get the long description from the README file
with open(path.join(path.abspath(path.dirname(__file__)), 'README.md'), encoding='utf-8') as f:
    LONG_DESCRIPTION = f.read()

# Arguments marked as "Required" below must be included for upload to PyPI.
# Fields marked as "Optional" may be commented out.

setup(
    name='apypie',
    version='0.2.0',
    description='Apipie bindings for Python',
    license='MIT',
    long_description=LONG_DESCRIPTION,
    long_description_content_type='text/markdown',
    url='https://github.com/Apipie/apypie',
    author='Evgeni Golov',
    author_email='evgeni@golov.de',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    keywords='api apipie bindings',
    packages=find_packages(exclude=['contrib', 'docs', 'tests']),
    install_requires=[
        'requests>=2.4.2',
    ],
)
